package com.ruoyi.system.utils.dbsync.runner;

import com.ruoyi.system.utils.dbsync.meta.Column;
import java.util.Map;
import java.util.List;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.SQLException;
import com.ruoyi.system.utils.dbsync.meta.Table;
import java.util.ArrayList;
import com.ruoyi.system.utils.dbsync.compare.CompareUnits;
import com.ruoyi.system.utils.dbsync.util.SqlUtil;
import com.ruoyi.system.utils.dbsync.meta.MetaData;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.mysql.cj.util.StringUtils;
import java.util.Properties;

public class Syncer
{
    public static void sync(final Properties prop) {
        final String sourceHost = prop.getProperty("sourceHost");
        if (StringUtils.isEmptyOrWhitespaceOnly(sourceHost)) {
            System.out.println("sourceHost is miss");
            System.exit(-1);
        }
        final String sourceUser = prop.getProperty("sourceUser");
        if (StringUtils.isEmptyOrWhitespaceOnly(sourceUser)) {
            System.out.println("sourceUser is miss");
            System.exit(-1);
        }
        final String sourcePass = prop.getProperty("sourcePass");
        if (StringUtils.isEmptyOrWhitespaceOnly(sourcePass)) {
            System.out.println("sourcePass is miss");
            System.exit(-1);
        }
        final String sourceSchema = prop.getProperty("sourceSchema");
        if (StringUtils.isEmptyOrWhitespaceOnly(sourceSchema)) {
            System.out.println("sourceSchema is miss");
            System.exit(-1);
        }
        final String sourceCharset = prop.getProperty("sourceCharset");
        if (StringUtils.isEmptyOrWhitespaceOnly(sourceCharset)) {
            System.out.println("sourceCharset is miss");
            System.exit(-1);
        }
        final String targetHost = prop.getProperty("targetHost");
        if (StringUtils.isEmptyOrWhitespaceOnly(targetHost)) {
            System.out.println("targetHost is miss");
            System.exit(-1);
        }
        final String targetUser = prop.getProperty("targetUser");
        if (StringUtils.isEmptyOrWhitespaceOnly(targetUser)) {
            System.out.println("targetUser is miss");
            System.exit(-1);
        }
        final String targetPass = prop.getProperty("targetPass");
        if (StringUtils.isEmptyOrWhitespaceOnly(targetPass)) {
            System.out.println("targetPass is miss");
            System.exit(-1);
        }
        final String targetSchema = prop.getProperty("targetSchema");
        if (StringUtils.isEmptyOrWhitespaceOnly(targetSchema)) {
            System.out.println("targetSchema is miss");
            System.exit(-1);
        }
        final String targetCharset = prop.getProperty("targetCharset");
        if (StringUtils.isEmptyOrWhitespaceOnly(targetCharset)) {
            System.out.println("targetCharset is miss");
            System.exit(-1);
        }
        String autoExecute = prop.getProperty("autoExecute");
        if (StringUtils.isEmptyOrWhitespaceOnly(autoExecute)) {
            autoExecute = "FALSE";
        }
        if (!sourceCharset.equals(targetCharset)) {
            System.out.println("source target charset not equal");
            System.out.println(-1);
        }
        final DbConfig config = new DbConfig();
        config.setSourceHost(sourceHost);
        config.setSourceUser(sourceUser);
        config.setSourcePass(sourcePass);
        config.setSourceSchema(sourceSchema);
        config.setSourceCharset(sourceCharset);
        config.setTargetHost(targetHost);
        config.setTargetUser(targetUser);
        config.setTargetPass(targetPass);
        config.setTargetSchema(targetSchema);
        config.setTargetCharset(targetCharset);
        doSync(config, (String)null);
    }
    
    public static String doSync(final String sourceHost, final String sourcePass, final String targetHost, final String targetPass, final String clientVersion) {
        final DbConfig config = new DbConfig();
        config.setSourceHost(sourceHost);
        config.setSourceUser("root");
        config.setSourcePass(sourcePass);
        config.setSourceSchema("sync-client");
        config.setSourceCharset("utf8");
        config.setTargetHost(targetHost);
        config.setTargetUser("root");
        config.setTargetPass(targetPass);
        config.setTargetSchema("sync-client");
        config.setTargetCharset("utf8");
        return doSync(config, clientVersion);
    }
    
    public static AjaxResult executeSql(final String host, final String pass, final String[] sql) {
        try {
            final MetaData target = new MetaData();
            target.setJdbcUrl("jdbc:mysql://" + host + "/sync-client?characterEncoding=utf8");
            target.setUser("root");
            target.setPassword(pass);
            target.setSchema("sync-client");
            if (!target.connect()) {
                return AjaxResult.error("客户端数据库连接失败，请检查3306端口是否设置白名单");
            }
            SqlUtil.executeBatchSqlWithTransaction(target.getConn(), sql);
            return AjaxResult.success();
        }
        catch (final Exception var4) {
            final Exception e = var4;
            return AjaxResult.error("执行失败 " + e.getMessage());
        }
    }
    
    public static String doSync(final DbConfig config, final String clientVersion) {
        final MetaData source = new MetaData();
        source.setJdbcUrl("jdbc:mysql://" + config.getSourceHost() + "/" + config.getSourceSchema() + "?characterEncoding=" + config.getSourceCharset());
        source.setUser(config.getSourceUser());
        source.setPassword(config.getSourcePass());
        source.setSchema(config.getSourceSchema());
        if (!source.init()) {
            return "源数据库连接失败";
        }
        final MetaData target = new MetaData();
        target.setJdbcUrl("jdbc:mysql://" + config.getTargetHost() + "/" + config.getTargetSchema() + "?characterEncoding=" + config.getTargetCharset());
        target.setUser(config.getTargetUser());
        target.setPassword(config.getTargetPass());
        target.setSchema(config.getTargetSchema());
        if (!target.init()) {
            return "目标数据库连接失败";
        }
        final CompareUnits units = new CompareUnits(source, target);
        units.compare();
        final StringBuilder sql = new StringBuilder();
        for (int i = 0; i < units.getChangeSql().size(); ++i) {
            sql.append(units.getChangeSql().get(i)).append("\n");
            SqlUtil.ddl(target.getConn(), (String)units.getChangeSql().get(i));
        }
        final String[] tables = { "la_system_auth_menu", "la_system_auth_perm", "la_system_auth_role" };
        sql.append(copyTableData(source, target, tables));
        return sql.toString();
    }
    
    public static String copyTableData(final MetaData source, final MetaData target, final String[] tableNames) {
        final Connection sourceConnection = source.getConn();
        final Connection targetConnection = target.getConn();
        PreparedStatement sourceStatement = null;
        PreparedStatement targetStatement = null;
        ResultSet resultSet = null;
        final StringBuilder result = new StringBuilder();
        try {
            targetConnection.setAutoCommit(false);
            final String[] var33 = tableNames;
            for (int var34 = tableNames.length, var35 = 0; var35 < var34; ++var35) {
                final String tableName = var33[var35];
                final List<String> columnNames = new ArrayList<String>();
                final StringBuilder insertSQL = new StringBuilder("INSERT INTO " + tableName + " (");
                final StringBuilder placeholders = new StringBuilder();
                final Map<String, Column> map = target.getTables().get(tableName).getColumns();
                map.values().forEach(Syncer::lambda$copyTableData$0);
                insertSQL.setLength();
                placeholders.setLength();
                insertSQL.append(") VALUES (").append((CharSequence)placeholders).append(")");
                final String selectSQL = "SELECT * FROM " + tableName;
                sourceStatement = sourceConnection.prepareStatement(selectSQL);
                resultSet = sourceStatement.executeQuery();
                targetStatement = targetConnection.prepareStatement(insertSQL.toString());
                final String dropTableSQL = "DELETE FROM " + tableName;
                final PreparedStatement delStatement = targetConnection.prepareStatement(dropTableSQL);
                delStatement.executeUpdate();
                while (resultSet.next()) {
                    for (int i = 0; i < columnNames.size(); ++i) {
                        targetStatement.setObject(i + 1, resultSet.getObject(columnNames.get(i)));
                    }
                    targetStatement.addBatch();
                }
                targetStatement.executeBatch();
                result.append("\n表数据复制成功: ").append(tableName);
            }
            targetConnection.commit();
        }
        catch (final Exception var36) {
            final Exception e = var36;
            try {
                targetConnection.rollback();
            }
            catch (final SQLException var37) {
                final SQLException ex = var37;
                throw new RuntimeException(ex);
            }
            e.printStackTrace();
            try {
                if (resultSet != null) {
                    resultSet.close();
                }
                if (sourceStatement != null) {
                    sourceStatement.close();
                }
                if (targetStatement != null) {
                    targetStatement.close();
                }
                if (sourceConnection != null) {
                    sourceConnection.close();
                }
                if (targetConnection != null) {
                    targetConnection.close();
                }
            }
            catch (final SQLException var38) {
                final SQLException e2 = var38;
                e2.printStackTrace();
            }
        }
        finally {
            try {
                if (resultSet != null) {
                    resultSet.close();
                }
                if (sourceStatement != null) {
                    sourceStatement.close();
                }
                if (targetStatement != null) {
                    targetStatement.close();
                }
                if (sourceConnection != null) {
                    sourceConnection.close();
                }
                if (targetConnection != null) {
                    targetConnection.close();
                }
            }
            catch (final SQLException var39) {
                final SQLException e3 = var39;
                e3.printStackTrace();
            }
        }
        return result.toString();
    }
    
    public static Boolean checkConnect(final String targetHost, final String targetPass) {
        final MetaData target = new MetaData();
        target.setJdbcUrl("jdbc:mysql://" + targetHost + "/sync-client?characterEncoding=utf8");
        target.setUser("root");
        target.setPassword(targetPass);
        target.setSchema("utf8");
        return target.connect();
    }
}
